<?php

namespace Drupal\custom_configuration\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\custom_configuration\Helper\ConfigurationHelper;
use Drupal\Core\Form\drupal_set_message;
use Drupal\Core\Path\CurrentPathStack;

/**
 * Class Edit Configuration.
 *
 * @package Drupal\custom_configuration\Form
 */
class EditConfiguration extends ConfigFormBase {

  protected $database;
  protected $configHelper;
  protected $path;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $con, ConfigurationHelper $helper, CurrentPathStack $path) {
    $this->database = $con;
    $this->configHelper = $helper;
    $this->path = $path;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
        $container->get('database'), $container->get('custom.configuration'), $container->get('path.current')
    );
  }

  /**
   * Get Editable of the config name.
   */
  protected function getEditableConfigNames() {
    return ['custom_configuration.edit_configuration'];
  }

  /**
   * Get the form id.
   */
  public function getFormId() {
    return 'custom_configuration_edit_configuration';
  }

  /**
   * Building a custom form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $current_path = $this->path->getPath();
    $config_id = explode('/', $current_path);
    $tempConfig = $config_id[count($config_id) - 1];
    if ($tempConfig) {
      $resultSet = $this->database->select('custom_configuration', 'cc')
        ->fields('cc', [
          'custom_config_name',
          'custom_config_value',
          'custom_config_status',
        ])
        ->condition('custom_config_id', $tempConfig)
        ->execute()->fetchObject();
      if ($resultSet) {
        $form['custom_configuration'] = [
          '#type' => 'detail',
          '#title' => $this->t('Edit Custom Configuration'),
          '#open' => TRUE,
        ];
        $form['custom_configuration']['edit_configuration'] = [
          '#type' => 'detail',
          '#title' => $this->t('Edit Configuration'),
        ];
        $form['custom_configuration']['edit_configuration']['key'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Key name'),
          '#maxlength' => 50,
          '#required' => TRUE,
          '#default_value' => $resultSet->custom_config_name,
        ];
        $form['custom_configuration']['edit_configuration']['value'] = [
          '#type' => 'textarea',
          '#title' => $this->t('Value'),
          '#required' => TRUE,
          '#rows' => 10,
          '#cols' => 58,
          '#attributes' => ['style' => 'width:auto'],
          '#default_value' => $resultSet->custom_config_value,
        ];
        $form['custom_configuration']['add_configuration']['status'] = [
          '#type' => 'select',
          '#title' => $this->t('Configuration Status'),
          '#options' => [
            '0' => $this->t('Inactive'),
            '1' => $this->t('Active'),
          ],
          '#default_value' => $resultSet->custom_config_status,
        ];
        $form['custom_configuration']['add_configuration']['edit_submission'] = [
          '#type' => 'submit',
          '#value' => $this->t('Save Configuration'),
        ];
        $form['custom_configuration']['add_configuration']['cancel_submission'] = [
          '#type' => 'submit',
          '#value' => $this->t('Cancel'),
        ];
      }
      else {
        drupal_set_message($this->t('Configuration does not exists.'), 'error');
        $form['cancel'] = [
          '#type' => 'submit',
          '#value' => $this->t('Cancel'),
        ];
      }
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('op')->getUntranslatedString() === 'Save Configuration') {
      $current_path = $this->path->getPath();
      $config_id = explode('/', $current_path);
      $tempConfig = $config_id[count($config_id) - 1];
      $post = [];
      $post['config_id'] = $tempConfig;
      $post['name'] = $form_state->getValue('key');
      $post['config_value'] = $form_state->getValue('value');
      $post['status'] = $form_state->getValue('status');
      $msg = $this->configHelper->updateValue($post);
      drupal_set_message($this->t("@message", ['@message' => $msg['message']]), $msg['status']);
      $form_state->setRedirectUrl(Url::fromRoute('custom_configuration.configuration_list'));
    }
    else {
      $form_state->setRedirectUrl(Url::fromRoute('custom_configuration.configuration_list'));
    }
  }

}
