<?php

namespace Drupal\custom_configuration\Helper;

use Drupal\Core\Database\Connection;

/**
 * CRUD operation for the custom configuration.
 *
 * @package Drupal\custom_configuration\Helper
 */
class ConfigurationHelper {

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $conn) {
    $this->database = $conn;
  }

  /**
   * Create machine name. Replace all characters except alpha & number.
   *
   * @param string $name
   *   Name will check and replace the string.
   *
   * @return string
   *   It will return the machine name.
   */
  public function createMachineName($name) {
    $name = preg_replace('/[^a-z0-9 ]/', '', strtolower(trim($name)));
    $name = preg_replace('/\s+/', ' ', $name);
    return preg_replace('/[^a-zA-Z0-9]/', '_', $name);
  }

  /**
   * Save configuration.
   *
   * @param array $post
   *   It will hold the custom configuration value.
   *
   * @return array
   *   It will provide the message with the status.
   */
  public function createConfiguration(array $post) {
    $name = (isset($post['name'])) ? $post['name'] : NULL;
    $machine_name = (isset($post['machine_name'])) ? $post['machine_name'] : NULL;
    $config_value = (isset($post['config_value'])) ? $post['config_value'] : NULL;
    $status = (isset($post['status'])) ? $post['status'] : NULL;
    $time = time();
    /* If machine name empty create machine name from name. */
    if ((empty($machine_name)) && (!empty($name))) {
      $machine_name = $this->createMachineName($name);
    }
    /* Must have value $name, $machine_name and $config_value. */
    if ((!empty($name)) && (!empty($machine_name)) && (!empty($config_value))) {
      try {
        $this->database->insert('custom_configuration')
          ->fields([
            'custom_config_name' => trim($name),
            'custom_config_machine_name' => $machine_name,
            'custom_config_value' => trim($config_value),
            'custom_config_status' => $status,
            'custom_config_updated_date' => $time,
            'custom_config_created_date' => $time,
          ])->execute();
        return ['status' => 'status', 'message' => 'Configuration saved successfully.'];
      }
      catch (\Exception $e) {
        return ['status' => 'error', 'message' => 'Key name is already exists.'];
      }
    }
    else {
      return ['status' => 'error', 'message' => 'Required field can not empty'];
    }
  }

  /**
   * Configuration getting values.
   *
   * @param string $machine_name
   *   Machine name.
   * @param string $status
   *   Status of the configuration.
   *
   * @return array
   *   It will return the configuration value.
   */
  public function getValue($machine_name = NULL, $status = 1) {
    if ($machine_name === NULL || $status === 0) {
      return NULL;
    }
    else {
      return $this->getConfigList($machine_name, $status);
    }
  }

  /**
   * Configuration getting values.
   *
   * @param string $machine_name
   *   Machine name.
   * @param string $status
   *   Status of the configuration.
   *
   * @return array
   *   It will return the array of the custom configuration list.
   */
  public function getConfigList($machine_name = NULL, $status = NULL) {
    if ($machine_name === NULL) {
      $result = $this->database->select('custom_configuration', 'cc')
        ->fields('cc', [
          'custom_config_id',
          'custom_config_name',
          'custom_config_machine_name',
          'custom_config_value',
          'custom_config_status',
          'custom_config_updated_date',
        ])->execute()->fetchAll();
      return $result;
    }
    else {
      $result = $this->database->select('custom_configuration', 'cc')
        ->fields('cc', ['custom_config_value'])
        ->condition('custom_config_machine_name', $machine_name, '=')
        ->condition('custom_config_status', $status, '=')
        ->execute()->fetchAssoc();
      return (count($result) > 0) ? $result['custom_config_value'] : NULL;
    }
  }

  /**
   * Delete the config id.
   *
   * @param int $config_id
   *   It will give the config id.
   *
   * @return array
   *   Return the status and message.
   */
  public function deleteValue($config_id) {
    $del = $this->database->delete('custom_configuration')
      ->condition('custom_config_id', $config_id)->execute();
    if ($del > 0) {
      return ['status' => 'status', 'message' => "Configuration deleted successfully."];
    }
    else {
      return ['status' => 'error', 'message' => "Configuration cannot be deleted."];
    }
  }

  /**
   * Updating the value.
   *
   * @param array $post
   *   This will have the name, value and status in an array.
   *
   * @return array
   *   This  will have the status and message.
   */
  public function updateValue(array $post) {
    $name = (isset($post['name'])) ? $post['name'] : NULL;
    $config_value = (isset($post['config_value'])) ? $post['config_value'] : NULL;
    $status = (isset($post['status'])) ? $post['status'] : NULL;
    if ($name !== NULL && $config_value !== NULL && $status !== NULL) {
      $row_updated = $this->database->update('custom_configuration')
        ->fields([
          'custom_config_name' => $name,
          'custom_config_value' => $config_value,
          'custom_config_status' => $status,
          'custom_config_updated_date' => time(),
        ])->condition('custom_config_id', $post['config_id'])->execute();
      if ($row_updated > 0) {
        return ['status' => 'status', 'message' => 'Configuration saved successfully.'];
      }
      else {
        return ['status' => 'error', 'message' => 'Configuration cannot be saved.'];
      }
    }
    else {
      return ['status' => 'error', 'message' => 'Required field can not empty'];
    }
  }

}
